/** @file
 * Copyright(C) OASIS(R) 2007,2010. All Rights Reserved.
 * OASIS trademark, IPR and other policies apply.
 *
 * This file contains the SCA API functions declarations for implementing SCA components and clients.
 * @author SCA-C-C++ TC
 * @version 1.1
 * @date 4 March 2010
 */

#ifdef __cplusplus
extern "C" {
#endif

/**
 * Initialize a Reference before invoking any operations of the Reference.
 * @param referenceName Name of the Reference to initialize.
 * @param referenceToken Token to be used in subsequent SCAInvoke() calls.
 *                       This will be NULL if referenceName is not defined for the component.
 * @param compCode
 *                 - SCACC_OK, if the call is successful.
 *                 - SCACC_ERROR, otherwise see reason for details.
 * @param reason
 *               - SCA_SERVICE_UNAVAILABLE if no suitable service exists in the domain.
 *               - SCA_MULTIPLE_SERVICES if the reference is bound to multiple services.
 */
void SCAGetReference(wchar_t *referenceName, SCAREF *referenceToken, int *compCode, int *reason);

/**
 * Initialize a Reference that might be bound to multiple Services before invoking any operations
 * of the Reference.
 * @param referenceName Name of the Reference to initialize.
 * @param referenceTokens Array of tokens to be used in subsequent SCAInvoke() calls.
 *                        These will all be NULL if referenceName is not defined for the component.
 *                        Operations need to be invoked on each token in the array.
 * @param num_targets Number of tokens returned in the array.
 * @param compCode
 *                 - SCACC_OK, if the call is successful.
 *                 - SCACC_ERROR, otherwise see reason for details.
 * @param reason SCA_SERVICE_UNAVAILABLE if no suitable service exists in the domain.
 */
void SCAGetReferences(wchar_t *referenceName,
                      SCAREF **referenceTokens,
                      int *num_targets,
                      int *compCode,
                      int *reason);

/**
 * Invoke an operation of an interface.
 * @param token Token returned by prior SCAGetReference() or SCAGetReferences(), SCASelf()
 *              or SCAGetCallback() call.
 * @param operationName Name of the operation to invoke.
 * @param inputMsgLen Length of the request message buffer.
 * @param inputMsg Request message.
 * @param outputMsgLen
 *                     - Input: Maximum number of bytes that can be returned.
 *                     - Output: Actual number of bytes returned or size needed to hold entire message.
 * @param outputMsg Response message.
 * @param compCode
 *                 - SCACC_OK, if the call is successful.
 *                 - SCACC_WARNING, if the response data was truncated.
 *                                  The buffer size needs to be increased and SCAGetReplyMessage()
 *                                  needs to be called with the larger buffer.
 *                 - SCACC_FAULT, if the operation returned a business fault.
 *                                SCAGetFaultMessage() should be called to get the fault details.
 *                 - SCACC_ERROR, otherwise see reason for details.
 * @param reason
 *               - SCA_DATA_TRUNCATED if the response data was truncated.
 *               - SCA_PARAMETER_ERROR if the operationName is not defined for the interface.
 *               - SCA_SERVICE_UNAVAILABLE if the provider for the interface is no longer operational.
 */
void SCAInvoke(SCAREF token,
               wchar_t *operationName,
               int inputMsgLen,
               void *inputMsg,
               int *outputMsgLen,
               void *outputMsg,
               int *compCode,
               int *reason);

/**
 * Get the configured value for a Property as a boolean.
 * @param propertyName Name of the Property value to obtain.
 * @param value Configured value(s) of the property.
 * @param num_values
 *                 - Input: Maximum number of configured values that can be returned.
 *                 - Output: Actual number of configured values.
 * @param compCode
 *                 - SCACC_OK, if the call is successful.
 *                 - SCACC_WARNING, if the number of configured values exceeds the input value of
 *                                  num_values.
 *                                  The call needs to be repeated wih value pointing to a location
 *                                  sufficient in size to hold all of the configured values.
 *                 - SCACC_ERROR, otherwise see reason for details.
 * @param reason
 *               - SCA_ADDITIONAL_VALUES, if the number of configured values exceeds the input value
 *                                        of num_values.
 *               - SCA_PARAMETER_ERROR, if the propertyName is not defined for the component.
 */
void SCAPropertyBoolean(wchar_t *propertyName, char *value, int *num_values, int *compCode, int *reason);

/**
 * Get the configured value for a Property as a byte.
 * @param propertyName Name of the Property value to obtain.
 * @param value Configured value(s) of the property.
 * @param num_values
 *                 - Input: Maximum number of configured values that can be returned.
 *                 - Output: Actual number of configured values.
 * @param compCode
 *                 - SCACC_OK, if the call is successful.
 *                 - SCACC_WARNING, if the number of configured values exceeds the input value of
 *                                  num_values.
 *                                  The call needs to be repeated wih value pointing to a location
 *                                  sufficient in size to hold all of the configured values.
 *                 - SCACC_ERROR, otherwise see reason for details.
 * @param reason
 *               - SCA_ADDITIONAL_VALUES, if the number of configured values exceeds the input value
 *                                        of num_values.
 *               - SCA_PARAMETER_ERROR, if the propertyName is not defined for the component.
 */
void SCAPropertyByte(wchar_t *propertyName, int8_t *value, int *num_values, int *compCode, int *reason);

/**
 * Get the configured value for a Property as an array of bytes.
 * @param propertyName Name of the Property value to obtain.
 * @param value Configured value(s) of the property.
 * @param size
 *             - Input: Maximum number of bytes that can be returned for each configured value.
 *             - Output: Actual number of bytes returned or size needed to hold a configured  value.
 *                       If the property is many valued, size is an array. On input only the first
 *                       value of the array is relevant; indicating the width of each member of the
 *                       value array. On return, for each returned configured value, the corresponding
 *                       value of size is the number of bytes of characters in the configured value.
 *                       If this number exceeds the input value, the configured value is truncated and
 *                       compCode and reason are set to indicate the data truncation.
 * @param num_values
 *                 - Input: Maximum number of configured values that can be returned.
 *                 - Output: Actual number of configured values.
 * @param compCode
 *                 - SCACC_OK, if the call is successful.
 *                 - SCACC_WARNING, if the data was truncated or the number of configured values exceeds
 *                                  the input value of num_values.
 *                 - SCACC_ERROR, otherwise see reason for details.
 * @param reason
 *               - SCA_ADDITIONAL_VALUES, if the number of configured values exceeds the input value
 *                                        of num_values. The call needs to be repeated wih value
 *                                        pointing to a location sufficient in size to hold all of the
 *                                        configured values.
 *               - SCA_DATA_TRUNCATED, if the data was truncated. The buffer size for each configured
 *                                     value needs to be increased and the call repeated with the
 *                                     larger buffer. If both the number of configured values exceeds
 *                                     the input value of num_values and some configured values was
 *                                     truncated, SCA_ADDITIONAL_VALUES is returned.
 *               - SCA_PARAMETER_ERROR, if the propertyName is not defined for the component.
 */
void SCAPropertyBytes(wchar_t *propertyName,
		              int8_t **value,
		              int *size,
		              int *num_values,
		              int *compCode,
		              int *reason);

/**
 * Get the configured value for a Property as a wide char.
 * @param propertyName Name of the Property value to obtain.
 * @param value Configured value(s) of the property.
 * @param num_values
 *                 - Input: Maximum number of configured values that can be returned.
 *                 - Output: Actual number of configured values.
 * @param compCode
 *                 - SCACC_OK, if the call is successful.
 *                 - SCACC_WARNING, if the number of configured values exceeds the input value of
 *                                  num_values.
 *                                  The call needs to be repeated wih value pointing to a location
 *                                  sufficient in size to hold all of the configured values.
 *                 - SCACC_ERROR, otherwise see reason for details.
 * @param reason
 *               - SCA_ADDITIONAL_VALUES, if the number of configured values exceeds the input value
 *                                        of num_values.
 *               - SCA_PARAMETER_ERROR, if the propertyName is not defined for the component.
 */
void SCAPropertyChar(wchar_t *propertyName, wchar_t *value, int *num_values, int *compCode, int *reason);

/**
 * Get the configured value for a Property as an array of wide chars (not necessarily null terminated).
 * @param propertyName Name of the Property value to obtain.
 * @param value Configured value(s) of the property.
 * @param size
 *             - Input: Maximum number of characters that can be returned for each configured value.
 *             - Output: Actual number of characters or size needed to hold a configured  value.
 *                       If the property is many valued, size is an array. On input only the first
 *                       value of the array is relevant; indicating the width of each member of the
 *                       value array. On return, for each returned configured value, the corresponding
 *                       value of size is the number of bytes of characters in the configured value.
 *                       If this number exceeds the input value, the configured value is truncated and
 *                       compCode and reason are set to indicate the data truncation.
 * @param num_values
 *                 - Input: Maximum number of configured values that can be returned.
 *                 - Output: Actual number of configured values.
 * @param compCode
 *                 - SCACC_OK, if the call is successful.
 *                 - SCACC_WARNING, if the data was truncated or the number of configured values exceeds
 *                                  the input value of num_values.
 *                 - SCACC_ERROR, otherwise see reason for details.
 * @param reason
 *               - SCA_ADDITIONAL_VALUES, if the number of configured values exceeds the input value
 *                                        of num_values. The call needs to be repeated wih value
 *                                        pointing to a location sufficient in size to hold all of the
 *                                        configured values.
 *               - SCA_DATA_TRUNCATED, if the data was truncated. The buffer size for each configured
 *                                     value needs to be increased and the call repeated with the
 *                                     larger buffer. If both the number of configured values exceeds
 *                                     the input value of num_values and some configured values was
 *                                     truncated, SCA_ADDITIONAL_VALUES is returned.
 *               - SCA_PARAMETER_ERROR if the propertyName is not defined for the component.
 */
void SCAPropertyChars(wchar_t *propertyName,
		              wchar_t **value,
		              int *size,
		              int *num_values,
		              int *compCode,
		              int *reason);

/**
 * Get the configured value for a Property as a char.
 * @param propertyName Name of the Property value to obtain.
 * @param value Configured value(s) of the property.
 * @param num_values
 *                 - Input: Maximum number of configured values that can be returned.
 *                 - Output: Actual number of configured values.
 * @param compCode
 *                 - SCACC_OK, if the call is successful.
 *                 - SCACC_WARNING, if the number of configured values exceeds the input value of
 *                                  num_values.
 *                                  The call needs to be repeated wih value pointing to a location
 *                                  sufficient in size to hold all of the configured values.
 *                 - SCACC_ERROR, otherwise see reason for details.
 * @param reason
 *               - SCA_ADDITIONAL_VALUES, if the number of configured values exceeds the input value
 *                                        of num_values.
 *               - SCA_PARAMETER_ERROR, if the propertyName is not defined for the component or its
 *                                      type is incompatible with char.
 */
void SCAPropertyCChar(wchar_t *propertyName, char *value, int *num_values, int *compCode, int *reason);

/**
 * Get the configured value for a Property as an array of chars (not necessarily null terminated).
 * @param propertyName Name of the Property value to obtain.
 * @param value Configured value(s) of the property.
 * @param size
 *             - Input: Maximum number of characters that can be returned for each configured value.
 *             - Output: Actual number of characters or size needed to hold a configured  value.
 *                       If the property is many valued, size is an array. On input only the first
 *                       value of the array is relevant; indicating the width of each member of the
 *                       value array. On return, for each returned configured value, the corresponding
 *                       value of size is the number of bytes of characters in the configured value.
 *                       If this number exceeds the input value, the configured value is truncated and
 *                       compCode and reason are set to indicate the data truncation.
 * @param num_values
 *                 - Input: Maximum number of configured values that can be returned.
 *                 - Output: Actual number of configured values.
 * @param compCode
 *                 - SCACC_OK, if the call is successful.
 *                 - SCACC_WARNING, if the data was truncated or the number of configured values exceeds
 *                                  the input value of num_values.
 *                 - SCACC_ERROR, otherwise see reason for details.
 * @param reason
 *               - SCA_ADDITIONAL_VALUES, if the number of configured values exceeds the input value
 *                                        of num_values. The call needs to be repeated wih value
 *                                        pointing to a location sufficient in size to hold all of the
 *                                        configured values.
 *               - SCA_DATA_TRUNCATED, if the data was truncated. The buffer size for each configured
 *                                     value needs to be increased and the call repeated with the
 *                                     larger buffer. If both the number of configured values exceeds
 *                                     the input value of num_values and some configured values was
 *                                     truncated, SCA_ADDITIONAL_VALUES is returned.
 *               - SCA_PARAMETER_ERROR if the propertyName is not defined for the component or its type
 *                                     is incompatible with char.
 */
void SCAPropertyCChars(wchar_t *propertyName,
		               char **value,
		               int *size,
		               int *num_values,
		               int *compCode,
		               int *reason);

/**
 * Get the configured value for a Property as a short.
 * @param propertyName Name of the Property value to obtain.
 * @param value Configured value(s) of the property.
 * @param num_values
 *                 - Input: Maximum number of configured values that can be returned.
 *                 - Output: Actual number of configured values.
 * @param compCode
 *                 - SCACC_OK, if the call is successful.
 *                 - SCACC_WARNING, if the number of configured values exceeds the input value of
 *                                  num_values.
 *                                  The call needs to be repeated wih value pointing to a location
 *                                  sufficient in size to hold all of the configured values.
 *                 - SCACC_ERROR, otherwise see reason for details.
 * @param reason
 *               - SCA_ADDITIONAL_VALUES, if the number of configured values exceeds the input value
 *                                        of num_values.
 *               - SCA_PARAMETER_ERROR, if the propertyName is not defined for the component.
 */
void SCAPropertyShort(wchar_t *propertyName, int16_t *value, int *num_values, int *compCode, int *reason);

/**
 * Get the configured value for a Property as a long.
 * @param propertyName Name of the Property value to obtain.
 * @param value Configured value(s) of the property.
 * @param num_values
 *                 - Input: Maximum number of configured values that can be returned.
 *                 - Output: Actual number of configured values.
 * @param compCode
 *                 - SCACC_OK, if the call is successful.
 *                 - SCACC_WARNING, if the number of configured values exceeds the input value of
 *                                  num_values.
 *                                  The call needs to be repeated wih value pointing to a location
 *                                  sufficient in size to hold all of the configured values.
 *                 - SCACC_ERROR, otherwise see reason for details.
 * @param reason
 *               - SCA_ADDITIONAL_VALUES, if the number of configured values exceeds the input value
 *                                        of num_values.
 *               - SCA_PARAMETER_ERROR, if the propertyName is not defined for the component.
 */
void SCAPropertyInt(wchar_t *propertyName, int32_t *value, int *num_values, int *compCode, int *reason);

/**
 * Get the configured value for a Property as a long long.
 * @param propertyName Name of the Property value to obtain.
 * @param value Configured value(s) of the property.
 * @param num_values
 *                 - Input: Maximum number of configured values that can be returned.
 *                 - Output: Actual number of configured values.
 * @param compCode
 *                 - SCACC_OK, if the call is successful.
 *                 - SCACC_WARNING, if the number of configured values exceeds the input value of
 *                                  num_values.
 *                                  The call needs to be repeated wih value pointing to a location
 *                                  sufficient in size to hold all of the configured values.
 *                 - SCACC_ERROR, otherwise see reason for details.
 * @param reason
 *               - SCA_ADDITIONAL_VALUES, if the number of configured values exceeds the input value
 *                                        of num_values.
 *               - SCA_PARAMETER_ERROR, if the propertyName is not defined for the component.
 */
void SCAPropertyLong(wchar_t *propertyName, int64_t *value, int *num_values, int *compCode, int *reason);

/**
 * Get the configured value for a Property as a float.
 * @param propertyName Name of the Property value to obtain.
 * @param value Configured value(s) of the property.
 * @param num_values
 *                 - Input: Maximum number of configured values that can be returned.
 *                 - Output: Actual number of configured values.
 * @param compCode
 *                 - SCACC_OK, if the call is successful.
 *                 - SCACC_WARNING, if the number of configured values exceeds the input value of
 *                                  num_values.
 *                                  The call needs to be repeated wih value pointing to a location
 *                                  sufficient in size to hold all of the configured values.
 *                 - SCACC_ERROR, otherwise see reason for details.
 * @param reason
 *               - SCA_ADDITIONAL_VALUES, if the number of configured values exceeds the input value
 *                                        of num_values.
 *               - SCA_PARAMETER_ERROR, if the propertyName is not defined for the component or its
 *                                      type is incompatible with float.
 */
void SCAPropertyFloat(wchar_t *propertyName, float *value, int *num_values, int *compCode, int *reason);

/**
 * Get the configured value for a Property as a double.
 * @param propertyName Name of the Property value to obtain.
 * @param value Configured value(s) of the property.
 * @param num_values
 *                 - Input: Maximum number of configured values that can be returned.
 *                 - Output: Actual number of configured values.
 * @param compCode
 *                 - SCACC_OK, if the call is successful.
 *                 - SCACC_WARNING, if the number of configured values exceeds the input value of
 *                                  num_values.
 *                                  The call needs to be repeated wih value pointing to a location
 *                                  sufficient in size to hold all of the configured values.
 *                 - SCACC_ERROR, otherwise see reason for details.
 * @param reason
 *               - SCA_ADDITIONAL_VALUES, if the number of configured values exceeds the input value
 *                                        of num_values.
 *               - SCA_PARAMETER_ERROR, if the propertyName is not defined for the component or its
 *                                      type is incompatible with double.
 */
void SCAPropertyDouble(wchar_t *propertyName, double *value, int *num_values, int *compCode, int *reason);

/**
 * Get the configured value for a Property as a string of wide chars.
 * @param propertyName Name of the Property value to obtain.
 * @param value Configured value(s) of the property.
 * @param size
 *             - Input: Maximum number of characters that can be returned for each configured value.
 *             - Output: Actual number of characters or size needed to hold a configured  value.
 *                       If the property is many valued, size is an array. On input only the first
 *                       value of the array is relevant; indicating the width of each member of the
 *                       value array. On return, for each returned configured value, the corresponding
 *                       value of size is the number of bytes of characters in the configured value.
 *                       If this number exceeds the input value, the configured value is truncated and
 *                       compCode and reason are set to indicate the data truncation.
 * @param num_values
 *                 - Input: Maximum number of configured values that can be returned.
 *                 - Output: Actual number of configured values.
 * @param compCode
 *                 - SCACC_OK, if the call is successful.
 *                 - SCACC_WARNING, if the data was truncated or the number of configured values exceeds
 *                                  the input value of num_values.
 *                 - SCACC_ERROR, otherwise see reason for details.
 * @param reason
 *               - SCA_ADDITIONAL_VALUES, if the number of configured values exceeds the input value
 *                                        of num_values. The call needs to be repeated wih value
 *                                        pointing to a location sufficient in size to hold all of the
 *                                        configured values.
 *               - SCA_DATA_TRUNCATED, if the data was truncated. The buffer size for each configured
 *                                     value needs to be increased and the call repeated with the
 *                                     larger buffer. If both the number of configured values exceeds
 *                                     the input value of num_values and some configured values was
 *                                     truncated, SCA_ADDITIONAL_VALUES is returned.
 *               - SCA_PARAMETER_ERROR if the propertyName is not defined for the component or its type
 *                                     is incompatible with a wchar_t string.
 */
void SCAPropertyString(wchar_t *propertyName,
		               wchar_t **value,
		               int *size,
		               int *num_values,
		               int *compCode,
		               int *reason);

/**
 * Get the configured value for a Property as a string of chars.
 * @param propertyName Name of the Property value to obtain.
 * @param value Configured value(s) of the property.
 * @param size
 *             - Input: Maximum number of characters that can be returned for each configured value.
 *             - Output: Actual number of characters or size needed to hold a configured  value.
 *                       If the property is many valued, size is an array. On input only the first
 *                       value of the array is relevant; indicating the width of each member of the
 *                       value array. On return, for each returned configured value, the corresponding
 *                       value of size is the number of bytes of characters in the configured value.
 *                       If this number exceeds the input value, the configured value is truncated and
 *                       compCode and reason are set to indicate the data truncation.
 * @param num_values
 *                 - Input: Maximum number of configured values that can be returned.
 *                 - Output: Actual number of configured values.
 * @param compCode
 *                 - SCACC_OK, if the call is successful.
 *                 - SCACC_WARNING, if the data was truncated or the number of configured values exceeds
 *                                  the input value of num_values.
 *                 - SCACC_ERROR, otherwise see reason for details.
 * @param reason
 *               - SCA_ADDITIONAL_VALUES, if the number of configured values exceeds the input value
 *                                        of num_values. The call needs to be repeated wih value
 *                                        pointing to a location sufficient in size to hold all of the
 *                                        configured values.
 *               - SCA_DATA_TRUNCATED, if the data was truncated. The buffer size for each configured
 *                                     value needs to be increased and the call repeated with the
 *                                     larger buffer. If both the number of configured values exceeds
 *                                     the input value of num_values and some configured values was
 *                                     truncated, SCA_ADDITIONAL_VALUES is returned.
 *               - SCA_PARAMETER_ERROR if the propertyName is not defined for the component or its type
 *                                     is incompatible with char string.
 */
void SCAPropertyCString(wchar_t *propertyName,
		                char **value,
		                int *size,
		                int *num_values,
		                int *compCode,
		                int *reason);

/**
 * Get the configured value for a Property as a pointer to a struct.
 * @param propertyName Name of the Property value to obtain.
 * @param value Configured value(s) of the property.
 * @param num_values
 *                 - Input: Maximum number of configured values that can be returned.
 *                 - Output: Actual number of configured values.
 * @param compCode
 *                 - SCACC_OK, if the call is successful.
 *                 - SCACC_WARNING, if the number of configured values exceeds the input value of
 *                                  num_values.
 *                                  The call needs to be repeated wih value pointing to a location
 *                                  sufficient in size to hold all of the configured values.
 *                 - SCACC_ERROR, otherwise see reason for details.
 * @param reason
 *               - SCA_ADDITIONAL_VALUES, if the number of configured values exceeds the input value
 *                                        of num_values.
 *               - SCA_PARAMETER_ERROR, if the propertyName is not defined for the component.
 */
void SCAPropertyStruct(wchar_t *propertyName, void *value, int *num_values, int *compCode, int *reason);

/**
 * Get the configured value for a Property as a DataObject.
 * @param propertyName Name of the Property value to obtain.
 * @param value Configured value(s) of the property.
 * @param num_values
 *                 - Input: Maximum number of configured values that can be returned.
 *                 - Output: Actual number of configured values.
 * @param compCode
 *                 - SCACC_OK, if the call is successful.
 *                 - SCACC_WARNING, if the number of configured values exceeds the input value of
 *                                  num_values.
 *                                  The call needs to be repeated wih value pointing to a location
 *                                  sufficient in size to hold all of the configured values.
 *                 - SCACC_ERROR, otherwise see reason for details.
 * @param reason
 *               - SCA_ADDITIONAL_VALUES, if the number of configured values exceeds the input value
 *                                        of num_values.
 *               - SCA_PARAMETER_ERROR, if the propertyName is not defined for the component.
 */
void SCAPropertyDataObject(wchar_t *propertyName,
		                   DATAOBJECT *value,
		                   int *num_values,
		                   int *compCode,
		                   int *reason);

/**
 * Retrieve the reply message of an operation invocation if the length of the message exceeded the
 * buffer size provided on SCAInvoke().
 * @param token Token returned by prior SCAGetReference() or SCAGetReferences(), SCASelf()
 *              or SCAGetCallback() call.
 * @param bufferLen
 *                  - Input: Maximum number of bytes that can be returned.
 *                  - Output: Actual number of bytes returned or size needed to hold entire message.
 * @param buffer Response message.
 * @param compCode
 *                 - SCACC_OK, if the call is successful.
 *                 - SCACC_WARNING, if the response data was truncated.
 *                                  The buffer size needs to be increased and the call repeated with the
 *                                  larger buffer.
 *                 - SCACC_ERROR, otherwise see reason for details.
 * @param reason SCA_DATA_TRUNCATED if the response data was truncated.
 */
void SCAGetReplyMessage(SCAREF token, int *bufferLen, void *buffer, int *compCode, int *reason);

/**
 * Retrieve the details of a business fault received in response to an operation invocation.
 * @param token Token returned by prior SCAGetReference() or SCAGetReferences(), SCASelf()
 *              or SCAGetCallback() call.
 * @param bufferLen
 *                  - Input: Maximum number of bytes that can be returned.
 *                  - Output: Actual number of bytes returned or size needed to hold entire message.
 * @param faultName Name of the business fault.
 * @param buffer Fault message.
 * @param compCode
 *                 - SCACC_OK, if the call is successful.
 *                 - SCACC_WARNING, if the response data was truncated.
 *                                  The buffer size needs to be increased and the call repeated with the
 *                                  larger buffer.
 *                 - SCACC_ERROR, otherwise see reason for details.
 * @param reason
 *               - SCA_DATA_TRUNCATED if the response data was truncated.
 *               - SCA_PARAMETER_ERROR if the last operation invoked on the Reference did not return a
 *                                     business fault.
 */
void SCAGetFaultMessage(SCAREF token,
                        int *bufferLen,
                        wchar_t **faultName,
                        void *buffer,
                        int *compCode,
                        int *reason);

/**
 * Return a business fault in response to a request.
 * @param serviceName Name of the Service of the component for which the fault is being returned.
 * @param operationName Name of the operation of the Service for which the fault is being returned.
 * @param faultName Name of the business fault
 * @param bufferLen Length of the fault message buffer.
 * @param buffer Fault message.
 * @param compCode
 *                 - SCACC_OK, if the call is successful.
 *                 - SCACC_ERROR, otherwise see reason for details.
 * @param reason
 *               - SCA_DATA_TRUNCATED if the response data was truncated.
 *               -  SCA_PARAMETER_ERROR if the serviceName is not defined for the component,
 *                                     operationName is not defined for the Service or the faultName is
 *                                     not defined for the operation.
 */
void SCASetFaultMessage(wchar_t *serviceName,
                        wchar_t *operationName,
                        wchar_t *faultName,
                        int bufferLen,
                        void *buffer,
                        int *compCode,
                        int *reason);

/**
 * Set up to call other operations of a service provided by a component.
 * @fn void SCASelf(wchar_t *serviceName, SCAREF *serviceToken, int *compCode, int *reason)
 * @param serviceName Name of the Service to access. If a component only provides one service,
 *                    this string can be empty.
 * @param serviceToken Token to be used in subsequent calls.
 * @param compCode
 *                 - SCACC_OK, if the call is successful.
 *                 - SCACC_ERROR, otherwise see reason for details.
 * @param reason
 *               - SCA_DATA_TRUNCATED if the response data was truncated.
 *               - SCA_PARAMETER_ERROR if the serviceName is not defined for the component.
 */
void SCASelf(wchar_t *serviceName, SCAREF *serviceToken, int *compCode, int *reason);

/**
 * Initialize a Reference before invoking any operations of the Reference.
 * @param serviceName Name of the Service to initialize.  If a component only provides one service,
 *                    this string can be empty.
 * @param serviceToken Token to be used in subsequent SCAInvoke() calls.
 *                     This will be NULL if serviceName is not defined for the component.
 * @param compCode
 *                 - SCACC_OK, if the call is successful.
 *                 - SCACC_ERROR, otherwise see reason for details.
 * @param reason SCA_SERVICE_UNAVAILABLE if client is no longer available in the domain.
 */
void SCAGetCallback(wchar_t *serviceName, SCAREF *serviceToken, int *compCode, int *reason);

/**
 * Tell the SCA runtime it has completed callback processing and the EndPointrReference can be released.
 * @param serviceToken Token returned by prior SCAGetCallback() call.
 * @param compCode
 *                 - SCACC_OK, if the call is successful.
 *                 - SCACC_ERROR, otherwise see reason for details.
 * @param reason SCA_SERVICE_UNAVAILABLE if the serviceToken is not valid.
 */
void SCAReleaseCallback(SCAREF serviceToken, int *compCode, int *reason);

/**
 * Invoke an operation of an interface.
 * @param token Token returned by prior SCAGetReference() or SCAGetReferences(), SCASelf()
 *              or SCAGetCallback() call.
 * @param operationName Name of the operation to invoke.
 * @param inputMsgLen Length of the request message buffer.
 * @param inputMsg Request message.
 * @param handler Address of the function to handle the asynchronous response.
 * @param compCode
 *                 - SCACC_OK, if the call is successful.
 *                 - SCACC_ERROR, otherwise see reason for details.
 * @param reason
 *               - SCA_BUSY if an operation is already outstanding for this Reference or Callback.
 *               - SCA_PARAMETER_ERROR if the operationName is not defined for the interface.
 *               - SCA_SERVICE_UNAVAILABLE if the provider for the interface is no longer operational.
 */
void SCAInvokeAsync(SCAREF token,
                    wchar_t *operationName,
                    int inputMsgLen,
                    void *inputMsg,
                    void (*handler)(short),
                    int *compCode,
                    int *reason);

/**
 * Invoke a long running operation of a Reference using the polling style.
 * @param token Token returned by prior SCAGetReference() or SCAGetReferences(), SCASelf()
 *              or SCAGetCallback() call.
 * @param operationName Name of the operation to invoke.
 * @param inputMsgLen Length of the request message buffer.
 * @param inputMsg Request message.
 * @param compCode
 *                 - SCACC_OK, if the call is successful.
 *                 - SCACC_ERROR, otherwise see reason for details.
 * @param reason
 *               - SCA_BUSY if an operation is already outstanding for this Reference or Callback.
 *               - SCA_PARAMETER_ERROR if the operationName is not defined for the interface.
 *               - SCA_SERVICE_UNAVAILABLE if the provider for the interface is no longer operational.
 */
void SCAInvokePoll(SCAREF token,
                   wchar_t *operationName,
                   int inputMsgLen,
                   void *inputMsg,
                   int *compCode,
                   int *reason);

/**
 * Determine if a response to a long running operation request has been received.
 * @fn void SCACheckResponse(SCAREF token, short *responseType, int *compCode, int *reason)
 * @param token Token returned by prior SCALocate(), SCALocateMultiple(), SCASelf()
 *              or SCAGetCallback() call.
 * @param responseType Type of response received.
 * @param compCode
 *                 - SCACC_OK, if the call is successful.
 *                 - SCACC_ERROR, otherwise see reason for details.
 * @param reason SCA_PARAMETER_ERROR if there is no outstanding operation for this Reference.
 */
void SCACheckResponse(SCAREF token, short *responseType, int *compCode, int *reason);

/**
 * Tell the SCA runtime it has completed callback processing and the EndPointrReference can be released.
 * @param token Token returned by prior SCALocate(), SCALocateMultiple(), SCASelf()
 *              or SCAGetCallback() call.
 * @param compCode
 *                 - SCACC_OK, if the call is successful.
 *                 - SCACC_ERROR, otherwise see reason for details.
 * @param reason SCA_PARAMETER_ERROR if there is no outstanding operation for this Reference.
 */
void SCACancelInvoke(SCAREF token, int *compCode, int *reason);

/**
 * Access a Service before invoking any operations of the Service.
 * @param serviceURI URI of the Service to access.
 * @param domainURI URI of the SCA domain.
 * @param serviceToken Token to be used in subsequent SCAInvoke() calls.
 *                       This will be NULL if the Service cannot be found.
 * @param compCode
 *                 - SCACC_OK, if the call is successful.
 *                 - SCACC_ERROR, otherwise see reason for details.
 * @param reason SCA_SERVICE_UNAVAILABLE if the domain does not exist of the service does not exist
 *                                       in the domain
 */
void SCAEntryPoint(wchar_t *serviceURI,
                   wchar_t *domainURI,
                   SCAREF *serviceToken,
                   int *compCode,
                   int *reason);


#ifdef __cplusplus
}
#endif
